<%-- $Id: edit_subtype.jsp 6613 2014-11-21 11:47:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@since 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.ItemSubtypeFileType"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.Metadata"
	import="net.sf.basedb.core.FileStoreEnabled"
	import="net.sf.basedb.core.DataFileType"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.json.JsonConverter"
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.Set"
	import="java.util.List"
	import="org.json.simple.JSONArray"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.ITEMSUBTYPE;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	ItemSubtype subtype = null;
	ItemQuery<ItemSubtypeFileType> fileTypesQuery = null;
	
	if (itemId == 0)
	{
		title="Create item subtype";
		cc.removeObject("item");
	}
	else
	{
		subtype = ItemSubtype.getById(dc, itemId);
		cc.setObject("item", subtype);
		title = "Edit item subtype -- " + HTML.encodeTags(subtype.getName());
		subtype.checkPermission(Permission.WRITE);
		fileTypesQuery = subtype.getDataFileTypes();
	}
	
	JSONObject jsonFileTypes = new JSONObject();
	jsonFileTypes.put("itemType", "DATAFILETYPE");
	
	if (fileTypesQuery != null)
	{
		fileTypesQuery.include(Include.ALL);
		fileTypesQuery.order(Orders.asc(Hql.property("itemSubtype.name")));
		jsonFileTypes.put("items", JsonUtil.toArray(fileTypesQuery.iterate(dc), new JsonConverter<ItemSubtypeFileType>() 
		{
			public Object convert(ItemSubtypeFileType ft)
			{
				JSONObject json = new JSONObject();
				DataFileType dft = ft.getDataFileType();
				boolean required = ft.isRequired();
				boolean multiple = ft.getAllowMultiple();
				int value = 0;
				if (required) value += 1;
				if (multiple) value += 2;
				json.put("id", dft.getId());
				json.put("name", dft.getName() + (required ? " [×]" : " [-]"));
				json.put("value", value);
				return json;
			}
		}));
	}

	Set<Item> subtypableItems = Metadata.getSubtypableItems();
	JSONObject jsonSubtypeableItems = new JSONObject();
	JSONArray jsonItemTypes = new JSONArray();
	for (Item item : subtypableItems)
	{
		jsonItemTypes.add(item.name());
		JSONObject jsonSt = new JSONObject();
		jsonSt.put("value", item.getValue());
		jsonSt.put("fileStoreEnabled", FileStoreEnabled.class.isAssignableFrom(item.getItemClass()) ? 1 : 0);
		jsonSt.put("supportsPushAnnotations", ItemSubtype.supportsPushAnnotations(item) ? 1 : 0);
		JSONArray jsonRelated = new JSONArray();
		for (Item related : ItemSubtype.getRelatedItems(item))
		{
			jsonRelated.add(related.name());
		}
		jsonSt.put("related", jsonRelated);
		jsonSubtypeableItems.put(item.name(), jsonSt);
	}

	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), subtype);
	ExtensionsInvoker invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,linkitems-2.js,~subtypes.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		<form action="index.jsp?ID=<%=ID%>" method="post" name="subtype">
		<input type="hidden" name="cmd" value="UpdateItem">

		<div id="page-data" class="datacontainer"
			data-subtype-id="<%=itemId %>"
			data-main-item-type="<%=subtype != null ? subtype.getMainItemType().name() : "" %>"
			data-item-types=<%=HTML.encodeTags(jsonItemTypes.toJSONString()) %>
			data-item-info="<%=HTML.encodeTags(jsonSubtypeableItems.toJSONString())%>"
		></div>

		<t:tabcontrol id="settings" 
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=subtype != null%>"
			extensions="<%=invoker%>">
		<t:tab id="info" title="Item subtype" helpid="itemsubtype.edit">
			<table class="fullform input100">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=subtype == null ? "focus-select" : "focus" %>"
					type="text" name="name" 
					value="<%=HTML.encodeTags(subtype == null ? Values.getString(cc.getPropertyValue("name"), "New item subtype") : subtype.getName())%>" 
					maxlength="<%=ItemSubtype.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Main item type</th>
				<td>
					<%
					if (subtype == null)
					{
						%>
						<select name="itemType" id="itemType" class="required unchangeable selectionlist">
						<%
						int itemCode = Values.getInt(cc.getPropertyValue("itemType"), -1);
						for (Item item : subtypableItems)
						{
							String selected = itemCode == item.getValue() ? "selected" : "";
							%>
							<option value="<%=item.name()%>" <%=selected%>><%=item.toString() %>
							<%
						}
						%>
						</select>
						<%
					}
					else
					{
						%>
						<%=subtype.getMainItemType() %>
						<%
					}
					%>
				</td>
				<td></td>
			</tr>
			<tr id="section.pushAnnotations">
				<th></th>
				<td><input type="checkbox" name="pushAnnotations" 
					id="pushAnnotations" value="1" 
					<%=(subtype != null && subtype.getPushAnnotations()) || 
						(subtype == null && Values.getBoolean(cc.getPropertyValue("pushAnnotations"))) ? "checked" : ""%>
					><label for="pushAnnotations">Push annotations to parent</label></td>
				<td></td>
			</tr>
			<tr class="big">
				<th>Description</th>
				<td>
					<textarea class="text" rows="4" name="description" id="description"
						><%=HTML.encodeTags(subtype == null ? cc.getPropertyValue("description") : subtype.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			<tr>
				<th>Related subtypes</th>
				<td><div id="section.none" style="display: none;"><i>- none -</i></div></td>
				<td></td>
			</tr>
			<%
			for (Item item : subtypableItems)
			{
				%>
				<tr id="section.<%=item.name()%>" style="display: none;">
					<th class="subprompt"><%=item%></th>
					<td>
						<base:select 
							id="<%="related."+item.name()%>" 
							data-item-type="<%=item.name()%>"
							current="<%=subtype == null ? null : subtype.getRelatedSubtype(item) %>"
							newitem="<%=subtype == null%>"
							clazz="selectionlist"
						/>
					</td>
					<td></td>
				</tr>
				<%
			}
			%>
			<tr class="dynamic">
				<th></th>
				<td colspan="2"></td>
			</tr>
			</table>
		</t:tab>
		<t:tab
			id="filetypes"
			title="File types"
			helpid="itemsubtype.filetypes">
			<div id="filetypes.enabled" style="height: 100%; display: none;">
				<table class="fullform input100">
				<tr class="dynamic">
					<th>File types</th>
					<td>
						<div class="selectionlist">
							<table>
							<tr>
							<td>
								<select name="fileTypes" id="fileTypes" 
									class="auto-init"
									data-auto-init="link-container"
									data-initial-items="[<%=HTML.encodeTags(jsonFileTypes.toJSONString()) %>]"
									size="10" multiple>
								</select>
							</td>
							<td style="vertical-align: top;">
								<base:buttongroup vertical="true">
									<base:button 
										id="btnAddFileTypes"
										subclass="leftaligned"
										style="width: 14em;"
										title="Add file types&hellip;" 
										tooltip="Add file types"
										/>
									<base:button 
										subclass="leftaligned auto-init"
										data-auto-init="remove-link"
										data-list-id="fileTypes"
										style="width: 14em;"
										title="Remove" 
										tooltip="Remove the selected file types"
									/>
								</base:buttongroup>
								<input type="checkbox" id="required" name="required" value="1">
									<label for="required">Required</label><br>
								<input type="checkbox" id="multiple" name="multiple" value="1">
									<label for="multiple">Allow multiple files</label>
								</td>
							</tr>
							</table>
						</div>
					</td>
				</tr>
				</table>
			</div>
			<div id="filetypes.disabled" class="bg-filled-50" style="height: 100%; display: none; border: 1px solid transparent;">
				<div class="messagecontainer error"></div>
			</div>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
